local FishWarden = FishWarden_79a6ca19_6666_4759_9b8f_a67708694e5b;

-- This is a list of the achievement ID numbers.
-- These MUST be "You must catch SPECIFIC fish/item" - i.e. it won't work with you need to fish from x pool etc..
-- Other achievements will be ignored, and may cause error - who knows
-- Achievements with special cases (i.e. returns 0 critera) have fishids put into SpcAchieveFilters
FishWarden.AchieveFilters =
    {
        726, -- Mr. Pinchy's Magical Crawdad Box
        878, -- One That Didn't Get Away
        1836, -- Old Crafty
        1837, -- Old Ironjaw
        5478, -- The Limnologist
        5479, -- The Oceanographer
        2094, -- A Penny For Your Thoughts
        2095, -- Silver in the City
        1957, -- There's Gold In That There Fountain
        1958, -- I Smell A Giant Rat
        3218, -- Turtles All the Way Down
    };

-- This is a list of requirements for achievements which fail to work as implemented above (i.e. return 0 critera)
-- The list is done so that k = int(AchieveID), v = {fishid1, fishid2, ...} - where v is an integer list, or a single int
FishWarden.SpcAchieveFilters =
    {
        [726] = 27388, -- Mr. Pinchy himself!
        [1836] = 34486,
        [1837] = 34484,
        [1958] = 43698,
        [3218] = 46109,
    };

-- The following is a list of Miscellaneous filters; note some of them do have special processing code (Quest, Price), but most don't.
-- If you feel you can understand the format of this list, feel free to add your own filters.
-- All you have to do is add a proper entry here, the GUI and processing code will automatically handle it.
-- This must be a list of tables with integer indices. Integer indices are implied in Lua when you don't specify the keys.
--
-- Each table in the list represents a Miscellaneous filter. They each must have the following two keys: 
--   name
--     This should be a UNIQUE name, it is the name of the saved variable
--     It should also be ONE WORD (no spaces)
--     It should be alphanumeric and must begin with a letter.
--   display
--     This is the description shown next to the check box in the GUI. It should be something that makes sense to the end-user.
--   Following those two values should be a simple list of itemIDs for the appropriate fish. Order isn't important, but the indices must be contiguous from [1..n]. Again this index behavior is implied when keys aren't specified.
--
-- Note: The order of this list is the order the GUI will display it in. If for some reason you want to rearrange it, it's easy copy-pasta.
--
-- Special Cases: These are filter names which are RESERVED special cases. You may delete or move them without issue. However, if you change the 'name' value, they will no longer function as expected - if at all.
--   FilterMiscQuest, FilterMiscItemLevel, FilterMiscPrice
FishWarden.OtherFilters =
    {
        {
            name = "FilterMiscQuest", --This list should contain "Quest Fish" which are not marked as "Quest Items" in the game. Most Quest Fish are marked as such.
            display = ITEM_BIND_QUEST,
            16967, --Feralas Ahi
            13757, --Lightning Eel
            13890, --Plated Armorfish
        },
        {name = "FilterMiscItemLevel", display = gsub(ITEM_LEVEL, "%%d", ">= ", 1),};
        {name = "FilterMiscPrice", display = SELL_PRICE .. " >= ",},
        {name = "FilterMiscFishFeast", display = (GetSpellInfo(57426)), 41806, 41809, 41813,},
        {name = "FilterMiscSeafoodFeast", display = (GetSpellInfo(88036)), 53064, 53068, 53070,},
        {
            name = "FilterMiscContainers",
            display = FishWarden.Locale.Containers,
            6356, --Battered Chest
            7973, --Big-mouth Clam
            35313, --Bloated Barbed Gill Trout
            6647, --Bloated Catfish
            35286, --Bloated Giant Sunfish
            45328, --Bloated Monsterbelly
            6645, --Bloated Mud Snapper
            13881, --Bloated Redgill
            13891, --Bloated Salmon
            45328, --Bloated Slippery Eel
            6643, --Bloated Smallfish
            8366, --Bloated Trout
            27513, --Curious Crate
            36781, --Darkwater Clam
            6351, --Dented Crate
            45909, --Giant Darkwater Clam
            13874, --Heavy Crate
            27481, --Heavy Supply Crate
            27511, --Inscribed Scrollcase
            21150, --Iron Bound Trunk
            13875, --Ironbound Locked Chest
            24476, --Jaggal Clam
            6307, --Message in a Bottle
            21228, --Mithril Bound Trunk
            44475, --Reinforced Crate
            6357, --Sealed Crate (35)
            67597, --Sealed Crate (85)
            6353, --Small Chest
            6354, --Small Locked Chest
            6355, --Sturdy Locked Chest
            20708, --Tightly Sealed Trunk
            6352, --Waterlogged Crate
            21113, --Watertight Trunk
        },
        {
            name = "FilterMiscEquippable",
            display = TUTORIAL_TITLE24,
            6292, --10 Pound Mud Snapper
            6294, --12 Pound Mud Snapper
            6295, --15 Pound Mud Snapper
            13901, --15 Pound Salmon
            13902, --18 Pound Salmon
            13903, --22 Pound Salmon
            13904, --25 Pound Salmon
            13905, --29 Pound Salmon
            6309, --17 Pound Catfish
            6310, --19 Pound Catfish
            6311, --22 Pound Catfish
            6363, --26 Pound Catfish
            6364, --32 Pound Catfish
            13885, --34 Pound Redgill
            13886, --37 Pound Redgill
            13882, --42 Pound Redgill
            13883, --45 Pound Redgill
            13884, --49 Pound Redgill
            13887, --52 Pound Redgill
            13914, --70 Pound Mightfish
            13915, --85 Pound Mightfish
            13916, --92 Pound Mightfish
            13917, --103 Pound Mightfish
            43659, --Bloodied Prison Shank
            6651, --Broken Wine Bottle
            44703, --Dark Herring
            6366, --Darkwood Fishing Pole
            44505, --Dustbringer
            27516, --Enormous Barbed Gill Trout
            16967, --Feralas Ahi
            27515, --Huge Spotted Feltail
            62065, --Melted Cleaver
            34486, --Old Crafty
            34484, --Old Ironjaw
            19808, --Rockhide Strongfish
            6360, --Steelscale Crushfish
            8350, --The 1 Ring
        },
        {name = "FilterMiscSTVContest", display = FishWarden.Locale.STVContest, 19807, 19805, 19806, 19803,},
        {name = "FilterMiscKaluakContest", display = FishWarden.Locale.KaluakContest, 50289,},
    };

