local FishWarden = FishWarden_79a6ca19_6666_4759_9b8f_a67708694e5b;

local MainFrame = FishWarden.GUI.MainFrame;
MainFrame:DoInit();
MainFrame:RegisterEvent("PLAYER_LOGIN");

local AchieveFrame = FishWarden.GUI.AchieveFrame;
local QualityFrame = FishWarden.GUI.QualityFrame;
local MiscFrame = FishWarden.GUI.MiscFrame;
local CustomFrame = FishWarden.GUI.CustomFrame;
local CustomHelpFrame = FishWarden.GUI.CustomHelpFrame;

SlashCmdList['FishWarden_GUI'] = FishWarden.GUI.MainFrame.Toggle;

_G['SLASH_FishWarden_GUI1'] = '/fishwarden';
_G['SLASH_FishWarden_GUI2'] = '/fward';

--[[
    Most of the tables used are cleared by setting each value to false when needed, instead of wiping the table
    This takes a little bit more CPU time; however, it saves on memory in the long run, and cuts down on garbage pile up
  ]]

-- This table will hold boolean variables, k = int(fishid), v = bool(needtocatch)
-- This is updated at loot time
FishWarden.AchieveFish = {};

-- This table will hold boolean variables, k = int(fishid), v = bool(needtocatch)
-- This is updated at loot time if the config changed at any point
-- OtherFish.Valid is not true when the list needs to be updated
FishWarden.OtherFish = {};

-- This table will hold integer variables, k = int(fishid), v = int(needtocatch)
-- needtocatch = {false = no(but check names), 0 = no(don't check names), 1 = yes, -1 = blacklisted}
-- This is updated at login, and if the Custom filter text changes (user presses 'Update')
FishWarden.CustomFish = {};

-- This table will hold integer variables, (k = string(texttosearch), v = int(needtocatch)) and (k = int(index), v = string(texttosearch))
-- index = contiguous from 1..n, and in the order that texttosearch appears in the custom filter text
-- needtocatch = {nil = no, 1 = yes, -1 = blacklisted}
-- This is updated at login, and if the Custom filter text changes (user presses 'Update')
-- This table is wiped on Update
FishWarden.CustomText = {};

-- This table will be a boolean list, k = int(loot slot #), v = needtoloot
-- This is updated at LOOT_OPEN, and per LOOT_SLOT_CLEARED so FishWarden can know when it's safe to close the loot window
-- This table grows only as large as ever needed, and never shrinks or wipes
-- LootSlots.NumSlots reflects how many slots were in the latest fishing loot window
FishWarden.LootSlots = {};

function FishWarden:PLAYER_LOGIN()
    if (not FishWardenDB) then
        FishWardenDB = {};
    end

    FishWarden.CharName = (UnitName("player")) .. " - " .. (GetRealmName());
    if (FishWardenDB[FishWarden.CharName]) then
        FishWarden.Vars = FishWardenDB[FishWarden.CharName];
    else
        FishWardenDB[FishWarden.CharName] = {};
        FishWarden.Vars = FishWardenDB[FishWarden.CharName];
    end

    --Defaults
    if (FishWarden.Vars.Enable == nil) then FishWarden.Vars.Enable = true; end
    if (FishWarden.Vars.FilterMiscQuest == nil) then FishWarden.Vars.FilterMiscQuest = true; end
    if (FishWarden.Vars.FilterMiscPriceVal == nil) then FishWarden.Vars.FilterMiscPriceVal = 10000; end
    if (FishWarden.Vars.FilterMiscMinItemLevel == nil) then FishWarden.Vars.FilterMiscMinItemLevel = 1; end
    if (not FishWarden.Vars.FilterAchieve) then
        FishWarden.Vars.FilterAchieve = {};
        for i, v in ipairs(FishWarden.AchieveFilters) do -- Transfer old variables to table.
            v = tonumber(v);
            if (v) then
                FishWarden.Vars.FilterAchieve[v] = FishWarden.Vars["FilterAchieve" .. v];
                FishWarden.Vars["FilterAchieve" .. v] = nil;
            end
        end
    end
    if (not FishWarden.Vars.FilterQuality) then
        FishWarden.Vars.FilterQuality = {};
        for i = 0, 7 do -- Transfer old variables to table.
            FishWarden.Vars.FilterQuality[i] = FishWarden.Vars["FilterQuality" .. i];
            FishWarden.Vars["FilterQuality" .. i] = nil;
        end
    end
    if (not FishWarden.Vars.FilterCustomText) then
        FishWarden.Vars.FilterCustomText = "";
    end

    -- Check if we need to reset Auto Loot after a camp or reload
    if (FishWarden.Vars.HadAutoLoot) then
        SetCVar("autoLootDefault", "1", AUTO_LOOT_DEFAULT_TEXT);
        FishWarden.Vars.HadAutoLoot = false;
    end

    -- Actually start doing addon stuff now
    if (FishWarden.Vars.Enable) then
        MainFrame:RegisterEvent("LOOT_OPENED");
    end

    FishWarden.itQuest = (select(10, GetAuctionItemClasses()));
    FishWarden.FishingSpell = GetSpellInfo(7620);

    InterfaceOptions_AddCategory(MainFrame);
    AchieveFrame:DoInit();
    QualityFrame:DoInit();
    MiscFrame:DoInit();
    CustomFrame:DoInit();
    CustomHelpFrame:DoInit();

    FishWarden:UpdateOtherFish();
    FishWarden:UpdateCustomFish();

    hooksecurefunc("TurnOrActionStart", FishWarden.TurnOrActionStart);
end

function FishWarden:SetEnable()
    FishWarden.Vars.Enable = not FishWarden.Vars.Enable;
    if (FishWarden.Vars.Enable) then
        if (not MainFrame:IsEventRegistered("LOOT_OPENED")) then
            MainFrame:RegisterEvent("LOOT_OPENED");
        end
    else
        if (MainFrame:IsEventRegistered("LOOT_OPENED")) then
            MainFrame:UnregisterEvent("LOOT_OPENED");
        end
    end
end

function FishWarden:SetAchieve(AchieveID)
    assert(AchieveID, "Bad call to SetAchieve.");
    AchieveID = tonumber(AchieveID);
    assert(AchieveID, "SetAchieve: Achievement ID could not be read as a number.");
    FishWarden.Vars.FilterAchieve[AchieveID] = not FishWarden.Vars.FilterAchieve[AchieveID];
end

function FishWarden:SetQuality(QualityID)
    assert(QualityID, "Bad call to SetQuality.");
    QualityID = tonumber(QualityID);
    assert(QualityID, "SetQuality: Quality ID was not a number.");
    FishWarden.Vars.FilterQuality[QualityID] = not FishWarden.Vars.FilterQuality[QualityID];
end

function FishWarden:UpdateCriteria()
    local curcount, curtype, curid, complete, _;
    for k, v in pairs(FishWarden.AchieveFish) do
        FishWarden.AchieveFish[k] = false;
    end

    for i, v in ipairs(FishWarden.AchieveFilters) do
        assert(v, "UpdateCriteria: Achievement Filter list is borked.");
        v = tonumber(v);
        assert(v, "UpdateCriteria: Achievement ID could not be read as a number.");
        complete = select(4, GetAchievementInfo(v));
        if (not complete and FishWarden.Vars.FilterAchieve[v]) then
            if (FishWarden.SpcAchieveFilters[v]) then
                local w = FishWarden.SpcAchieveFilters[v];
                if (type(w) == "number") then
                    FishWarden.AchieveFish[w] = true;
                elseif (type(w) == "table") then
                    for k, x in ipairs(w) do
                        FishWarden.AchieveFish[tonumber(x)] = true;
                    end
                else
                    assert(nil, "Invalid value type for Special Achievement Filter.")
                end
            else
                for j=1, GetAchievementNumCriteria(v) do
                    _, curtype, complete, _, _, _, _, curid = GetAchievementCriteriaInfo(v, j);
                    if (not complete and (curtype == 42 or curtype == 36)) then
                        FishWarden.AchieveFish[tonumber(curid)] = true;
                    end
                end
            end
        end
    end
end

function FishWarden:UpdateOtherFish()
    for k, v in pairs(FishWarden.OtherFish) do
        FishWarden.OtherFish[k] = false;
    end

    for i, v in ipairs(FishWarden.OtherFilters) do
        if (FishWarden.Vars[v.name]) then
            if (type(v) == "table") then
                for j, w in ipairs(v) do
                    FishWarden.OtherFish[tonumber(w)] = true;
                end
            else
                assert(nil, "OtherFilters list fail. Value is not a number or table.");
            end
        end
    end
    FishWarden.OtherFish.Valid = true;
end

local malformederr = "|cffffffffFish Warden - " .. CUSTOM .. ":|r " .. FishWarden.Locale.MalformedCustomText;
function FishWarden:UpdateCustomFish()
    local text = FishWarden.Vars.FilterCustomText;

    for k, v in pairs(FishWarden.CustomFish) do
        FishWarden.CustomFish[k] = false;
    end
    wipe(FishWarden.CustomText);

    if (not text) then return; end

    local start, length, finish = 1, strlen(text);
    local substring, splitter;
    local paren, hyphen, need, malformed = 0;
    local textindex = 0;
    
    while (start <= length) do
        finish = strfind(text, "[\r\n%:%#%(%)]", start) or (length+1);
        splitter = strbyte(text, finish);
        substring = strsub(text, start, finish-1);
        substring = strtrim(substring, " \t\"[]");

        hyphen = strfind(substring, "[^-]");
        if (hyphen) then
            substring = strsub(substring, hyphen);
            hyphen = hyphen - 1;
        else
            substring = "";
            hyphen = 0;
        end
        need = math.pow(-1, paren + hyphen);

        if (strfind(substring, "^=")) then -- substring starts with '='
            start = strfind(text, "=", start) + 1; -- we need to regain anything we lost by not reading until EOL
            finish = strfind(text, "[\r\n]", finish) or (length+1);
            splitter = strbyte(text, finish);
            substring = strsub(text, start, finish-1);
            textindex = textindex + 1;
            FishWarden.CustomText[textindex] = substring;
            FishWarden.CustomText[substring] = need;            
        elseif (strlen(substring) > 0) then
            hyphen = tonumber(substring); -- we no longer need hyphen, reuse it for itemid
            if (hyphen) then
                FishWarden.CustomFish[hyphen] = FishWarden.CustomFish[hyphen] or need; -- ignore duplicates to follow the "first match" rule
            else
                substring = strlower(substring);
                textindex = textindex + 1;
                FishWarden.CustomText[textindex] = substring;
                FishWarden.CustomText[substring] = need;
            end
        end

        if (splitter == 35) then -- '#'
            finish = strfind(text, "[\r\n]", finish) or length; -- respect comment until next new-line
        elseif (splitter == 40) then -- '('
            paren = paren + 1;
        elseif (splitter == 41) then -- ')'
            paren = paren - 1;
            if (paren < 0) then malformed = true; end
        end
        start = finish + 1;
    end

    if (FishWarden.Vars.Debug) then
        local ids, bids, names, bnames = 0, 0, 0, 0;
        for k, v in pairs(FishWarden.CustomFish) do
            if (type(v) == "number") then
                if (v > 0) then ids = ids + 1; end
                if (v < 0) then bids = bids + 1; end
            end
        end
        for k, v in pairs(FishWarden.CustomText) do
            if (type(v) == "number") then
                if (v > 0) then names = names + 1; end
                if (v < 0) then bnames = bnames + 1; end
            end
        end
        FishWarden.Debug("Added", ids, "item IDs and", names, "item names.");
        FishWarden.Debug("Blacklisted", bids, "item IDs and", bnames, "item names.");
    end

    if (malformed or paren ~= 0) then
        FishWarden:Warn(malformederr, "Sound\\Creature\\GnomeSpiderTank\\GnomeSpiderTankDeath.wav");
    end
end

function FishWarden:SearchCustomFish(itemname, itemid)
    local ret = FishWarden.CustomFish[itemid]; -- check item IDs/cache first
    if (ret) then return ret; end

    if (not itemname) then return false; end -- unfortunately we can't continue if we don't have the itemname

    local lowitemname = strlower(itemname);

    ret = 0;
    for i, v in ipairs(FishWarden.CustomText) do
        if (v and (strfind(itemname, v) or strfind(lowitemname, v, 1, true))) then
            ret = FishWarden.CustomText[v];
            break;
        end
    end

    FishWarden.CustomFish[itemid] = ret; -- cache this search
    return ret;
end

function FishWarden:TakeLoot(slot)
    if (FishWarden.Vars.AutoCloseLoot and not MainFrame:IsEventRegistered("LOOT_SLOT_CLEARED")) then
        MainFrame:RegisterEvent("LOOT_SLOT_CLEARED");
        MainFrame:RegisterEvent("LOOT_CLOSED");
    end
    FishWarden.LootSlots[slot] = 1;
    LootSlot(slot);
end

function FishWarden:PrintIgnore()
    local firstignore;
    local info = ChatTypeInfo["MONSTER_WHISPER"];

    for i=1, FishWarden.LootSlots.NumSlots do
        if (FishWarden.LootSlots[i] == -1) then
            firstignore = i;
            break;
        end
    end

    if (firstignore) then
          DEFAULT_CHAT_FRAME:AddMessage(
              format(LOOT_ROLL_PASSED, "Fish Warden", strjoin(", ", FishWarden:ListIgnore(firstignore))),
              info.r, info.g, info.b);
    end
end

function FishWarden:ListIgnore(curslot)
    if (curslot > FishWarden.LootSlots.NumSlots) then
        return;
    elseif (FishWarden.LootSlots[curslot] == -1) then
        return GetLootSlotLink(curslot), FishWarden:ListIgnore(curslot + 1);
    else
        return FishWarden:ListIgnore(curslot + 1);
    end
end

function FishWarden:CloseLoot()
    for i=1, FishWarden.LootSlots.NumSlots do
        if (FishWarden.LootSlots[i] == 1) then
            return;
        end
    end

    CloseLoot(); -- We reached the end of the list without finding an unlooted keeper
end

function FishWarden.TurnOrActionStart()
    local channel = UnitChannelInfo("player");
    if (FishWarden.Vars.Enable and channel == FishWarden.FishingSpell) then
        if (GetCVarBool("autoLootDefault")) then
            FishWarden.Vars.HadAutoLoot = true;
            SetCVar("autoLootDefault", "0", AUTO_LOOT_DEFAULT_TEXT);
        end
    elseif (FishWarden.Vars.HadAutoLoot) then
        SetCVar("autoLootDefault", "1", AUTO_LOOT_DEFAULT_TEXT);
        FishWarden.Vars.HadAutoLoot = false;
    end
end

function FishWarden:LOOT_OPENED(autolooting)
    if (not IsFishingLoot()) then
        return;
    end
    if (autolooting ~= 0) then
        FishWarden:Warn(FishWarden.Locale.AutoLootError, "Sound\\Creature\\GnomeSpiderTank\\GnomeSpiderTankDeath.wav");
        return;
    end

    FishWarden:UpdateCriteria();
--[[ Currently there's no easy way to see if we actually need to update achievement criteria.
     The code to step through each selected achievement to check if any new criteria has been completed
     would be more time consuming than just rebuilding the list each loot.
     To keep the memory usage down, the list never shrinks or wipes, the keys are integers, and the values are boolean
  ]]
    if (not FishWarden.OtherFish.Valid) then FishWarden:UpdateOtherFish(); end

    FishWarden.LootSlots.NumSlots = GetNumLootItems();
    local link, itemname, itemid, quality, itemlevel, itemtype, itemprice, _;
    for i=1, GetNumLootItems() do
        FishWarden.LootSlots[i] = -1;
        if (LootSlotIsCoin(i)) then
            FishWarden:TakeLoot(i); -- If can ever fish up coin (gold/silver/copper) currency just take it.
        elseif (LootSlotIsItem(i)) then
            link = GetLootSlotLink(i);
            if (link) then
                itemid = tonumber((select(2,strsplit(":", link))));
                _, itemname, _, quality = GetLootSlotInfo(i);
                itemlevel, _, itemtype, _, _, _, _, itemprice = select(4, GetItemInfo(itemid));

                -- We need to check the custom filter FIRST, in case of blacklisted items
                local iscustomfish = FishWarden.Vars.FilterCustomEnable and FishWarden:SearchCustomFish(itemname, itemid);
                if (iscustomfish and iscustomfish ~= 0) then
                    if (iscustomfish > 0) then
                        FishWarden:TakeLoot(i);
                    else
                        FishWarden.Debug("Blacklisted", link);
                    end
                elseif (FishWarden.Vars.FilterQuality[quality]) then FishWarden:TakeLoot(i);
                elseif (MiscFrame.Buttons.FilterMiscQuest and FishWarden.Vars.FilterMiscQuest and itemtype == FishWarden.itQuest) then FishWarden:TakeLoot(i);
                elseif (MiscFrame.Buttons.FilterMiscPrice and FishWarden.Vars.FilterMiscPrice and itemprice and itemprice >= FishWarden.Vars.FilterMiscPriceVal) then
                    FishWarden:TakeLoot(i);
                elseif (MiscFrame.Buttons.FilterMiscItemLevel and FishWarden.Vars.FilterMiscItemLevel and itemlevel and itemlevel >= FishWarden.Vars.FilterMiscMinItemLevel) then
                    FishWarden:TakeLoot(i);
                elseif (FishWarden.AchieveFish[itemid]) then FishWarden:TakeLoot(i);
                elseif (FishWarden.OtherFish[itemid]) then FishWarden:TakeLoot(i);
                end
            end
        end
    end

    if (FishWarden.Vars.AutoCloseLoot) then
        FishWarden:PrintIgnore();
        MainFrame:RegisterEvent("CHAT_MSG_ADDON"); -- We fire this immediately after LOOT_OPENED from fishing
        SendAddonMessage("Fish Warden", "RunCloseLoot", "WHISPER", (UnitName("player")));
    end
end

function FishWarden:CHAT_MSG_ADDON(prefix, message, channel, sender)
    -- Now every addon should've had a chance to process LOOT_OPENED, so let's see if we can close it
    if (prefix == "Fish Warden" and message == "RunCloseLoot" and channel == "WHISPER" and sender == (UnitName("player"))) then
        if (GetNumLootItems() > 0) then
            FishWarden:CloseLoot();
        end
        MainFrame:UnregisterEvent("CHAT_MSG_ADDON");
    end
end

function FishWarden:LOOT_SLOT_CLEARED(slot)
    if (IsFishingLoot() and FishWarden.Vars.AutoCloseLoot) then
        assert(slot, "Event LOOT_SLOT_CLEARED called without a slot number.");
        assert(type(slot) == "number", "Event LOOT_SLOT_CLEARED called with invalid slot type.");
        FishWarden.LootSlots[slot] = 0;
        FishWarden:CloseLoot();
    else
        MainFrame:UnregisterEvent("LOOT_SLOT_CLEARED");
    end
end

function FishWarden:LOOT_CLOSED()
    if (MainFrame:IsEventRegistered("LOOT_SLOT_CLEARED")) then
        MainFrame:UnregisterEvent("LOOT_SLOT_CLEARED");
    end
    MainFrame:UnregisterEvent("LOOT_CLOSED");
end

local LastWarn = "";
local LastWarnTime = -2;
function FishWarden:Warn(msg, sound)
    if ((msg ~= LastWarn) or (GetTime() - 2 >= LastWarnTime)) then
        LastWarn = msg;
        LastWarnTime = GetTime();
        RaidNotice_AddMessage(RaidBossEmoteFrame, msg, ChatTypeInfo["RAID_WARNING"]);
        PlaySoundFile(sound);
    end
end

